@@ -1,10 +1,10 @@
 package Module::ScanDeps;
-use 5.006;
+use 5.008001;
 use strict;
 use warnings;
 use vars qw( $VERSION @EXPORT @EXPORT_OK @ISA $CurrentPackage @IncludeLibs $ScanFileRE );
 
-$VERSION   = '0.98';
+$VERSION   = '1.07';
 @EXPORT    = qw( scan_deps scan_deps_runtime );
 @EXPORT_OK = qw( scan_line scan_chunk add_deps scan_deps_runtime path_to_inc_name );
 
@@ -266,7 +266,16 @@ my %Preload;
     'Crypt/Random/Generator.pm' => sub {
         _glob_in_inc('Crypt/Random/Provider', 1);
     },
+    'Date/Manip.pm' => [qw( Date/Manip/DM5.pm Date/Manip/DM6.pm )],
+    'Date/Manip/Base.pm' => sub {
+        _glob_in_inc('Date/Manip/Lang', 1);
+    },
+    'Date/Manip/TZ.pm' => sub {
+        return (_glob_in_inc('Date/Manip/TZ', 1),
+                _glob_in_inc('Date/Manip/Offset', 1));
+    },
     'DateTime/Locale.pm' => 'sub',
+    'DateTime/TimeZone.pm' => 'sub',
     'DBI.pm' => sub {
         grep !/\bProxy\b/, _glob_in_inc('DBD', 1);
     },
@@ -315,6 +324,7 @@ my %Preload;
     },
     'Log/Log4perl.pm' => 'sub',
     'Log/Any.pm' => 'sub',
+    'Log/Report/Dispatcher.pm' => 'sub',
     'LWP/UserAgent.pm' => sub {
         return(
             qw(
@@ -346,31 +356,52 @@ my %Preload;
         _glob_in_inc('$CurrentPackage/Plugin', 1);
     },
     'MIME/Decoder.pm'               => 'sub',
+    'Moose.pm'                      => sub {
+        _glob_in_inc('Moose', 1),
+        _glob_in_inc('Class/MOP', 1),
+    },
+    'MooseX/AttributeHelpers.pm'    => 'sub',
+    'MooseX/POE.pm'                 => sub {
+        _glob_in_inc('MooseX/POE', 1),
+        _glob_in_inc('MooseX/Async', 1),
+    },
+    'MozRepl.pm'                    => sub {
+        qw( MozRepl/Log.pm MozRepl/Client.pm Module/Pluggable/Fast.pm ),
+        _glob_in_inc('MozRepl/Plugin', 1),
+    },
     'Net/DNS/RR.pm'                 => 'sub',
     'Net/FTP.pm'                    => 'sub',
     'Net/Server.pm'                 => 'sub',
     'Net/SSH/Perl.pm'               => 'sub',
+    'Package/Stash.pm'              => [qw( Package/Stash/PP.pm Package/Stash/XS.pm )],
     'PAR/Repository.pm'             => 'sub',
     'PAR/Repository/Client.pm'      => 'sub',
+    'Parse/AFP.pm'                  => 'sub',
+    'Parse/Binary.pm'               => 'sub',
     'Perl/Critic.pm'                => 'sub', #not only Perl/Critic/Policy
+    'PerlIO.pm'                     => [ 'PerlIO/scalar.pm' ],
     'PDF/API2/Resource/Font.pm'     => 'sub',
     'PDF/API2/Basic/TTF/Font.pm'    => sub {
         _glob_in_inc('PDF/API2/Basic/TTF', 1);
     },
     'PDF/Writer.pm'                 => 'sub',
-    'POE.pm'                           => [ qw(
-        POE/Kernel.pm POE/Session.pm
-    ) ],
+    'POE.pm'                        => [qw( POE/Kernel.pm POE/Session.pm )],
+    'POE/Component/Client/HTTP.pm'  => sub {
+        _glob_in_inc('POE/Component/Client/HTTP', 1),
+        qw( POE/Filter/HTTPChunk.pm POE/Filter/HTTPHead.pm ),
+    },
     'POE/Kernel.pm'                    => sub {
         _glob_in_inc('POE/XS/Resource', 1),
         _glob_in_inc('POE/Resource', 1),
         _glob_in_inc('POE/XS/Loop', 1),
         _glob_in_inc('POE/Loop', 1),
     },
+    'POSIX.pm'                      => sub {
+        map { my $sigmod = $_;
+              map "auto/POSIX/$sigmod/$_->{name}", _glob_in_inc("auto/POSIX/$sigmod");
+            } qw( SigAction SigRt )
+    },
     'PPI.pm'                        => 'sub',
-    'Parse/AFP.pm'                  => 'sub',
-    'Parse/Binary.pm'               => 'sub',
-    'PerlIO.pm'                     => [ 'PerlIO/scalar.pm' ],
     'Regexp/Common.pm'              => 'sub',
     'RPC/XML/ParserFactory.pm'      => sub {
         _glob_in_inc('RPC/XML/Parser', 1);
@@ -379,8 +410,11 @@ my %Preload;
         termios.ph asm/termios.ph sys/termiox.ph sys/termios.ph sys/ttycom.ph
     ) ],
     'SOAP/Lite.pm'                  => sub {
-        (($] >= 5.008 ? ('utf8.pm') : ()), _glob_in_inc('SOAP/Transport', 1));
+        ($] >= 5.008 ? ('utf8.pm') : ()), 
+        _glob_in_inc('SOAP/Transport', 1),
+        _glob_in_inc('SOAP/Lite/Deserializer', 1),
     },
+    'Socket/GetAddrInfo.pm'         => 'sub',
     'SQL/Parser.pm' => sub {
         _glob_in_inc('SQL/Dialects', 1);
     },
@@ -412,10 +446,11 @@ my %Preload;
     'Tk/Toplevel.pm'    => [qw( Tk/Wm.pm )],
     'Unicode/UCD.pm'    => sub {
         # add data files (cf. sub openunicode in Unicode::UCD)
+        'unicore/version',
         grep /\.txt$/, map "unicore/$_->{name}", _glob_in_inc('unicore', 0);
     },
     'URI.pm'            => sub {
-        grep !/.\b[_A-Z]/, _glob_in_inc('URI', 1);
+        grep !/urn/, _glob_in_inc('URI', 1);
     },
     'Win32/EventLog.pm'    => [qw( Win32/IPC.pm )],
     'Win32/Exe.pm'         => 'sub',
@@ -469,37 +504,9 @@ my %Preload;
     # anybody using threads::shared is likely to declare variables
     # with attribute :shared
     'utf8.pm' => sub {
-        # NOTE: Do NOT indiscrimantly add every *.pl below unicore. This would
-        # be a major performance regression. A simple
-        #
-        #   scandeps -B -e 'use utf8;' 
-        #
-        # takes 9.8 seconds wallclock time with the current implementation,
-        # but a version which adds every *.pl file takes 24.6 seconds.
-        return 'utf8_heavy.pl', do {
-            my $dir = 'unicore';
-            my @subdirs = qw( To );
-            my @files = map "$dir/lib/$_->{name}", _glob_in_inc("$dir/lib");
-
-            if (@files) {
-                # 5.8.x, 5.10.x (Exact, Canonical), 5.12.x (Heavy)
-                push @files, (map "$dir/$_.pl", qw( Exact Canonical  Heavy ));
-            }
-            else {
-                # 5.6.x
-                $dir = 'unicode';
-                @files = map "$dir/Is/$_->{name}", _glob_in_inc("$dir/Is")
-                  or return;
-                push @subdirs, 'In';
-            }
-
-            foreach my $subdir (@subdirs) {
-                foreach (_glob_in_inc("$dir/$subdir")) {
-                    push @files, "$dir/$subdir/$_->{name}";
-                }
-            }
-            @files;
-        }
+        # Perl 5.6.x: "unicode", Perl 5.8.x and up: "unicore"
+        my $unicore = _find_in_inc('unicore/Name.pl') ? 'unicore' : 'unicode';
+        return ('utf8_heavy.pl', map "$unicore/$_->{name}", _glob_in_inc($unicore));
     },
     'charnames.pm' => sub {
         _find_in_inc('unicore/Name.pl') ? 'unicore/Name.pl' : 'unicode/Name.pl'
@@ -721,14 +728,9 @@ sub scan_deps_runtime {
     elsif ($execute) {
         my $excarray = (ref($execute)) ? $execute : [@$files];
         my $exc;
-        my $first_flag = 1;
         foreach $exc (@$excarray) {
             ($inchash, $dl_shared_objects, $incarray) = ({}, [], []);
-            _execute(
-                $perl, $exc, $inchash, $dl_shared_objects, $incarray,
-                $first_flag
-            );
-            $first_flag = 0;
+            _execute($perl, $exc, $inchash, $dl_shared_objects, $incarray);
         }
 
         # XXX only retains data from last execute ...  Why? I suspect
@@ -788,13 +790,15 @@ sub scan_line {
     $line =~ s/[\\\/]+/\//g;
 
     foreach (split(/;/, $line)) {
-        if (/^\s*package\s+(\w+)/) {
+        s/^\s*//;
+
+        if (/^package\s+(\w+)/) {
             $CurrentPackage = $1;
             $CurrentPackage =~ s{::}{/}g;
             return;
         }
         # use VERSION:
-        if (/^\s*(?:use|require)\s+v?(\d[\d\._]*)/) {
+        if (/^(?:use|require)\s+v?(\d[\d\._]*)/) {
           # include feature.pm if we have 5.9.5 or better
           if (version->new($1) >= version->new("5.9.5")) {
               # seems to catch 5.9, too (but not 5.9.4)
@@ -802,21 +806,25 @@ sub scan_line {
           }
         }
 
-        if (my ($autouse) = /^\s*use\s+autouse\s+(["'].*?["']|\w+)/)
+        if (my ($autouse) = /^use \s+ autouse \s+ (["'].*?["']|\w+)/x)
         {
             $autouse =~ s/["']//g;
             $autouse =~ s{::}{/}g;
             return ("autouse.pm", "$autouse.pm");
         }
 
-        if (my ($libs) = /\b(?:use\s+lib\s+|(?:unshift|push)\W+\@INC\W+)(.+)/)
+        if (my ($how, $libs) = /^(use \s+ lib \s+ | (?:unshift|push) \s+ \@INC \s+ ,) (.+)/x)
         {
             my $archname = defined($Config{archname}) ? $Config{archname} : '';
             my $ver = defined($Config{version}) ? $Config{version} : '';
-            foreach (grep(/\w/, split(/["';() ]/, $libs))) {
-                unshift(@INC, "$_/$ver")           if -d "$_/$ver";
-                unshift(@INC, "$_/$archname")      if -d "$_/$archname";
-                unshift(@INC, "$_/$ver/$archname") if -d "$_/$ver/$archname";
+            foreach my $dir (do { no strict; no warnings; eval $libs }) {
+                next unless defined $dir;
+                my @dirs = $dir;
+                push @dirs, "$dir/$ver", "$dir/$archname", "$dir/$ver/$archname" 
+                    if $how =~ /lib/;
+                foreach (@dirs) {
+                    unshift(@INC, $_) if -d $_;
+                }
             }
             next;
         }
@@ -828,27 +836,35 @@ sub scan_line {
 }
 
 # short helper for scan_chunk
-sub _typical_module_loader_chunk {
-  local $_ = shift;
-  my $loader = shift;
-  my $prefix='';
-  if (@_ and $_[0]) {
-    $prefix=$_[0].'::';
-  }
-  my $loader_file = $loader;
-  $loader_file =~ s/::/\//;
-  $loader_file .= ".pm";
-  $loader = quotemeta($loader);
+my %LoaderRegexp; # cache
+sub _build_loader_regexp {
+    my $loaders = shift;
+    my $prefix = (@_ && $_[0]) ? $_[0].'::' : '';
+   
+    my $loader = join '|', map quotemeta($_), split /\s+/, $loaders;
+    my $regexp = qr/^\s* use \s+ ($loader)(?!\:) \b \s* (.*)/sx;
+    # WARNING: This doesn't take the prefix into account
+    $LoaderRegexp{$loaders} = $regexp;
+    return $regexp
+}
+
+# short helper for scan_chunk
+sub _extract_loader_dependency {
+    my $loader = shift;
+    my $loadee = shift;
+    my $prefix = (@_ && $_[0]) ? $_[0].'::' : '';
+
+    my $loader_file = $loader;
+    $loader_file =~ s/::/\//;
+    $loader_file .= ".pm";
 
-  if (/^\s* use \s+ $loader(?!\:) \b \s* (.*)/sx) {
     return [
-      $loader_file,
-      map { my $mod="$prefix$_";$mod=~s{::}{/}g; "$mod.pm" }
-      grep { length and !/^q[qw]?$/ and !/-/ } split(/[^\w:-]+/, $1)
-      #should skip any module name that contains '-', not split it in two
+        $loader_file,
+        map { my $mod="$prefix$_"; $mod =~ s{::}{/}g; "$mod.pm" }
+        grep { length and !/^q[qw]?$/ and !/-/ }
+        split /[^\w:-]+/, $loadee
+        #should skip any module name that contains '-', not split it in two
     ];
-  }
-  return();
 }
 
 sub scan_chunk {
@@ -857,33 +873,38 @@ sub scan_chunk {
     # Module name extraction heuristics {{{
     my $module = eval {
         $_ = $chunk;
+        s/^\s*//;
 
         # TODO: There's many more of these "loader" type modules on CPAN!
         # scan for the typical module-loader modules
-        foreach my $loader (qw(asa base parent prefork POE encoding maybe only::matching)) {
-          my $retval = _typical_module_loader_chunk($_, $loader);
+        my $loaders = "asa base parent prefork POE encoding maybe only::matching";
+        # grab pre-calculated regexp or re-build it (and cache it)
+        my $loader_regexp = $LoaderRegexp{$loaders} || _build_loader_regexp($loaders);
+        if ($_ =~ $loader_regexp) { # $1 == loader, $2 == loadee
+          my $retval = _extract_loader_dependency($1, $2);
           return $retval if $retval;
         }
 
-        foreach my $loader (qw(Catalyst)) {
-          my $retval = _typical_module_loader_chunk($_, $loader,'Catalyst::Plugin');
+        $loader_regexp = $LoaderRegexp{"Catalyst"} || _build_loader_regexp("Catalyst", "Catalyst::Plugin");
+        if ($_ =~ $loader_regexp) { # $1 == loader, $2 == loadee
+          my $retval = _extract_loader_dependency($1, $2, "Catalyst::Plugin");
           return $retval if $retval;
         }
 
         return [ 'Class/Autouse.pm',
             map { s{::}{/}g; "$_.pm" }
               grep { length and !/^:|^q[qw]?$/ } split(/[^\w:]+/, $1) ]
-          if /^\s* use \s+ Class::Autouse \b \s* (.*)/sx
-              or /^\s* Class::Autouse \s* -> \s* autouse \s* (.*)/sx;
+          if /^use \s+ Class::Autouse \b \s* (.*)/sx
+              or /^Class::Autouse \s* -> \s* autouse \s* (.*)/sx;
 
-        return $1 if /(?:^|\s)(?:use|no|require)\s+([\w:\.\-\\\/\"\']+)/;
+        return $1 if /^(?:use|no|require) \s+ ([\w:\.\-\\\/\"\']+)/x;
         return $1
-          if /(?:^|\s)(?:use|no|require)\s+\(\s*([\w:\.\-\\\/\"\']+)\s*\)/;
+          if /^(?:use|no|require) \s+ \( \s* ([\w:\.\-\\\/\"\']+) \s* \)/x;
 
-        if (   s/(?:^|\s)eval\s+\"([^\"]+)\"/$1/
-            or s/(?:^|\s)eval\s*\(\s*\"([^\"]+)\"\s*\)/$1/)
+        if (   s/^eval\s+\"([^\"]+)\"/$1/
+            or s/^eval\s*\(\s*\"([^\"]+)\"\s*\)/$1/)
         {
-            return $1 if /(?:^|\s)(?:use|no|require)\s+([\w:\.\-\\\/\"\']*)/;
+            return $1 if /^\s* (?:use|no|require) \s+ ([\w:\.\-\\\/\"\']*)/x;
         }
 
         if (/(<[^>]*[^\$\w>][^>]*>)/) {
@@ -896,8 +917,8 @@ sub scan_chunk {
             return [ 'PerlIO.pm', $mod ] if $1 and $mod;
             return $mod if $mod;
         }
-        return $1 if /(?:^|\s)(?:do|require)\s+[^"]*"(.*?)"/;
-        return $1 if /(?:^|\s)(?:do|require)\s+[^']*'(.*?)'/;
+        return $1 if /^(?:do|require)\s+[^"]*"(.*?)"/;
+        return $1 if /^(?:do|require)\s+[^']*'(.*?)'/;
         return $1 if /[^\$]\b([\w:]+)->\w/ and $1 ne 'Tk' and $1 ne 'shift';
         return $1 if /\b(\w[\w:]*)::\w+\(/ and $1 ne 'main' and $1 ne 'SUPER';
 
@@ -1121,13 +1142,13 @@ sub new {
 
 sub set_file {
     my $self = shift;
-    foreach my $script (@_) {
-        my ($vol, $dir, $file) = File::Spec->splitpath($script);
-        $self->{main} = {
-            key  => $file,
-            file => $script,
-        };
-    }
+    my $script = shift;
+
+    my ($vol, $dir, $file) = File::Spec->splitpath($script);
+    $self->{main} = {
+        key  => $file,
+        file => $script,
+    };
 }
 
 sub set_options {
@@ -1211,52 +1232,70 @@ sub get_files {
 
 # scan_deps_runtime utility functions
 
-sub _compile {
-    my ($perl, $file, $inchash, $dl_shared_objects, $incarray) = @_;
+sub _compile    { _compile_or_execute(1, @_) }
+sub _execute    { _compile_or_execute(0, @_) }
 
-    my ($fhout, $fname) = File::Temp::tempfile("XXXXXX");
-    my $fhin  = FileHandle->new($file) or die "Couldn't open $file\n";
+sub _compile_or_execute {
+    my ($do_compile, $perl, $file, $inchash, $dl_shared_objects, $incarray) = @_;
 
-    my $line = do { local $/; <$fhin> };
-    $line =~ s/use Module::ScanDeps::DataFeed.*?\n//sg;
-    $line =~ s/^(.*?)((?:[\r\n]+__(?:DATA|END)__[\r\n]+)|$)/
-use Module::ScanDeps::DataFeed '$fname.out';
-sub {
-$1
-}
-$2/s;
-    $fhout->print($line);
-    $fhout->close;
-    $fhin->close;
+    require Module::ScanDeps::DataFeed; 
+    # ... so we can find it's full pathname in %INC
 
-    system($perl, $fname);
+    my ($feed_fh, $feed_file) = File::Temp::tempfile();
+    my $dump_file = "$feed_file.out";
 
-    _extract_info("$fname.out", $inchash, $dl_shared_objects, $incarray);
-    unlink("$fname");
-    unlink("$fname.out");
-}
+    require Data::Dumper;
 
-sub _execute {
-    my ($perl, $file, $inchash, $dl_shared_objects, $incarray, $firstflag) = @_;
+    # spoof $0 (to $file) so that FindBin works as expected
+    # NOTE: We don't directly assign to $0 as it has magic (i.e.
+    # assigning has side effects and may actually fail, cf. perlvar(1)).
+    # Instead we alias *0 to a package variable holding the correct value.
+    print $feed_fh "BEGIN { ", 
+                   Data::Dumper->Dump([ $file ], [ "Module::ScanDeps::DataFeed::_0" ]),
+                   "*0 = \\\$Module::ScanDeps::DataFeed::_0; }\n";
 
-    $DB::single = $DB::single = 1;
-    my ($fhout, $fname) = File::Temp::tempfile("XXXXXX");
-    $fname = _abs_path($fname);
-    my $fhin  = FileHandle->new($file) or die "Couldn't open $file";
+    print $feed_fh $do_compile ? "INIT {\n" : "END {\n";
+    # NOTE: When compiling the block will run _after_ all CHECK blocks
+    # (but _before_ the first INIT block) and will terminate the program.
+    # When executing the block will run as the first END block and 
+    # the programs continues.
 
-    my $line = do { local $/; <$fhin> };
-    $line =~ s/use Module::ScanDeps::DataFeed.*?\n//sg;
-    $line = "use Module::ScanDeps::DataFeed '$fname.out';\n" . $line;
-    $fhout->print($line);
-    $fhout->close;
-    $fhin->close;
+    # correctly escape strings containing filenames
+    print $feed_fh map { "my $_" } Data::Dumper->Dump(
+                       [ $INC{"Module/ScanDeps/DataFeed.pm"}, $dump_file ],
+                       [ qw( datafeedpm dump_file ) ]);
+
+    print $feed_fh <<'...';
+    # save %INC etc so that further requires dont't pollute them
+    %Module::ScanDeps::DataFeed::_INC = %INC;
+    @Module::ScanDeps::DataFeed::_INC = @INC;
+    @Module::ScanDeps::DataFeed::_dl_shared_objects = @DynaLoader::dl_shared_objects;
+    @Module::ScanDeps::DataFeed::_dl_modules = @DynaLoader::dl_modules;
+
+        require $datafeedpm;
+
+    Module::ScanDeps::DataFeed::_dump_info($dump_file);
+...
+
+    print $feed_fh $do_compile ? "exit(0);\n}\n" : "}\n";
+
+    # append the file to compile
+    {
+        open my $fhin, "<", $file or die "Couldn't open $file: $!";
+        print $feed_fh qq[#line 1 "$file"\n], <$fhin>;
+        close $fhin;
+    }
+    close $feed_fh;
 
     File::Path::rmtree( ['_Inline'], 0, 1); # XXX hack
-    system($perl, (map { "-I$_" } @IncludeLibs), $fname) == 0 or die "SYSTEM ERROR in executing $file: $?";
+    my $rc = system($perl, (map { "-I$_" } @IncludeLibs), $feed_file);
 
-    _extract_info("$fname.out", $inchash, $dl_shared_objects, $incarray);
-    unlink("$fname");
-    unlink("$fname.out");
+    _extract_info($dump_file, $inchash, $dl_shared_objects, $incarray) if $rc == 0;
+    unlink($feed_file, $dump_file);
+    die $do_compile
+            ? "SYSTEM ERROR in compiling $file: $rc" 
+            : "SYSTEM ERROR in executing $file: $rc" 
+            unless $rc == 0;
 }
 
 # create a new hashref, applying fixups
@@ -1271,8 +1310,7 @@ sub _make_rv {
 
     require File::Spec;
 
-    my $key;
-    foreach $key (keys(%$inchash)) {
+    foreach my $key (keys(%$inchash)) {
         my $newkey = $key;
         $newkey =~ s"^(?:(?:$inc)/?)""sg if File::Spec->file_name_is_absolute($newkey);
 
@@ -1284,8 +1322,7 @@ sub _make_rv {
         };
     }
 
-    my $dl_file;
-    foreach $dl_file (@$dl_shared_objects) {
+    foreach my $dl_file (@$dl_shared_objects) {
         my $key = $dl_file;
         $key =~ s"^(?:(?:$inc)/?)""s;
 
@@ -1304,11 +1341,11 @@ sub _extract_info {
     my ($fname, $inchash, $dl_shared_objects, $incarray) = @_;
 
     use vars qw(%inchash @dl_shared_objects @incarray);
-    my $fh = FileHandle->new($fname) or die "Couldn't open $fname";
-    my $line = do { local $/; <$fh> };
-    $fh->close;
 
-    eval $line;
+    unless (do $fname) {
+        die "error extracting info from DataFeed file: ",
+            $@ || "can't read $fname: $!";
+    }
 
     $inchash->{$_} = $inchash{$_} for keys %inchash;
     @$dl_shared_objects = @dl_shared_objects;
